/*
 *  Types/DataType.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */



using System;
using System.Collections;
using System.Xml;
using Gtk;


namespace Incollector.DataTypes {
	
	/// <summary>Managing data types</summary>
	public class DataTypes {
		static public DataType [] DataTypesList = new DataType [] {
						new Conversation (),
						new Note (),
						new Quote (),
						new SerialNumber (),
						new SourceCode (),
						new WebAddress (),
						new Word ()
					};
		
		
		/// <summary>Returns data type by name.</summary>
		/// <param name="name">Name of data type</param>
		static public DataType GetDataType (string name)
		{
			foreach (DataType tpe in DataTypesList)
				if (tpe.Name == name)
					return tpe.New ();
			
			return null;
		}
		
		
		static public int GetDataTypeID (string name)
		{
			int i = 0;
			foreach (DataType tpe in DataTypesList) {
				if (tpe.Name == name)
					return i;
				
				i++;
			}
			
			return -1;
		}
	}
	
	
	
	/// <summary>Describes a data type.</summary>
	public abstract class DataType {

		/// <summary>Returns name of data type</summary>
		/// <remarks>Name is short identifier of data type for DataTypesList. It is not displayed.</remarks>
		public abstract string Name {
			get ;
		}
		
		/// <summary>Returns title of data type</summary>
		/// <remarks>Title is full name of data type. It is used in GUI.</remarks>
		public abstract string Title {
			get ;
		}
		
		/// <summary>Returns data type menu item text. For example: Add a note.</summary>
		public abstract string MenuText {
			get ;
		}
		
		/// <summary>Returns a widget that can be focused default.</summary>
		public abstract Gtk.Widget DefaultFocusWidget {
			get ;
		}
		
		/// <summary>Returns a widget that can be focused.</summary>
		public abstract Gtk.Widget FocusWidget {
			get ;
		}
		
		public abstract Gtk.Widget EditWidget {
			get ;
		}
		
		public abstract string EditWidgetContent {
			set ;
		}
		
		public abstract Gtk.Widget PreviewWidget {
			get ;
		}
		
		public abstract string EntryTitle {
			get ;
		}
		
		/// <summary>Returns icon (16x16) pixbuf</summary>
		public abstract Gdk.Pixbuf Icon16 {
			get ;
		}
		
		/// <summary>Returns icon (32x32) pixbuf</summary>
		public abstract Gdk.Pixbuf Icon32 {
			get ;
		}
		
		/// <summary>Returns icon (48x48) pixbuf</summary>
		public abstract Gdk.Pixbuf Icon48 {
			get ;
		}
		
		
		
		public DataType ()
		{
		}
		
		
		public abstract DataType New ();
		public abstract DataType Clone ();
		
		/// <summary>Reads data from XML</summary>
		/// <returns>Entry with filled "data" field</returns>
		/// <param name="reader">XmlTextReader</param>
		/// <remarks>This XML code to save in entries database.</remarks>
		public abstract void ReadXml (XmlTextReader reader);
		
		/// <summary>Writes data to XML</summary>
		/// <param name="writer">XmlTextWriter</param>
		/// <remarks>This XML code to save in entries database.</remarks>
		public abstract void WriteXml (XmlTextWriter writer);
		
		/// <summary>Add "changed" EntryEditWidget event handler</summary>
		public abstract void AddChangedEvent (EventHandler hdlr);
		
		public void UnparentWidget ()
		{
			if (this.EditWidget.Parent is Gtk.Box)
				(this.EditWidget.Parent as Gtk.Box).Remove (this.EditWidget);
		}
		
		/// <summary>Search for "query"</summary>
		/// <returns>True, if data was found</returns>
		public abstract bool Search (string query);
		
		/// <summary>Returns true, if Edit entry is filled.</summary>
		public abstract bool IsFormFilledIn ();
		
		public abstract void SynchronizeData ();
	}
}
